from math import pi

from ..unit_converter import convert_unit


def air_weight(tvd, nominal_weight):
    """
    Calculate axial force due to pipe weight in air
    :param tvd: list - true vertical depth, m
    :param nominal_weight: weight per unit length, kg/m
    :return: axial force profile, kN
    """

    f_w = [nominal_weight * (tvd[-1] - x) / 1000 for x in tvd]

    return f_w


def buoyancy_force(tvd, od_csg, id_csg, tvd_fluid_ext, rho_fluid_ext, tvd_fluid_int, rho_fluid_int):
    """
    Calculate axial force due to buoyancy effect
    :param tvd: list - true vertical depth, m
    :param od_csg: pipe outer diameter, in
    :param id_csg: pipe inner diameter, in
    :param tvd_fluid_ext: list - reference tvd of fluid change outside, m
    :param rho_fluid_ext: list - downwards sorted fluids densities outside, sg
    :param tvd_fluid_int: list - reference tvd of fluid change inside, m
    :param rho_fluid_int: list - downwards sorted fluids densities inside, sg
    :return: axial force profile, kN
    """
    p_ext = pressure_profile(tvd, tvd_fluid_ext, rho_fluid_ext)
    p_int = pressure_profile(tvd, tvd_fluid_int, rho_fluid_int)

    area_total = (pi / 4) * od_csg ** 2
    area_total = convert_unit(area_total, unit_from="in2", unit_to="m2")

    area_int = (pi / 4) * id_csg ** 2
    area_int = convert_unit(area_int, unit_from="in2", unit_to="m2")

    f_bu = [area_total * x - area_int * y for x, y in zip(p_ext, p_int)]
    f_bu = [x/1000 for x in f_bu]       # N to kN

    return f_bu


def drag(trajectory, od_csg, id_csg, shoe_depth, nominal_weight, tvd_fluid, rho_fluid, sliding_fric=0.24,
         case='lowering', hole=10):
    """
    Calculate axial force due to drag effect
    :param trajectory: wellpath object
    :param od_csg: pipe outer diameter, in
    :param id_csg: pipe inner diameter, in
    :param shoe_depth: measured depth at shoe, m
    :param nominal_weight: weight per unit length, kg/m
    :param tvd_fluid: list - reference tvd of fluid change
    :param rho_fluid: list - downwards sorted fluids densities
    :param sliding_fric: sliding friction factor pipe - wellbore
    :param case: operational case "lowering", "static", "hoisting" or "all"
    :param hole: borehole size, in
    :return: axial force profile, kN
    """

    import torque_drag

    rhof = density_profile(trajectory.tvd, tvd_fluid, rho_fluid)

    area = (pi / 4) * (od_csg ** 2 - id_csg * 2)        # in2
    area = convert_unit(area, unit_from="in2", unit_to="m2")

    rho_pipe = convert_unit(nominal_weight/area, unit_from="kg/m3", unit_to="sg")    # kg/m3 to sg

    f_d = torque_drag.calc(trajectory,
                           dimensions={'od_pipe': od_csg,
                                       'id_pipe': id_csg,
                                       'od_annular': hole,
                                       'length_pipe': shoe_depth},
                           densities={'rhof': rhof, 'rhod': rho_pipe},
                           case=case,
                           fric=sliding_fric,
                           wob=0,
                           tbit=0).force[case]      # kN

    return f_d


def pressure_testing(tvd, whp, effective_diameter):
    """
    Calculate axial force due to pressure testing
    :param tvd: list - true vertical depth, m
    :param whp: wellhead pressure, bar
    :param effective_diameter: diameter, in
    :return: axial force profile, kN
    """

    whp = convert_unit(whp, unit_from="bar", unit_to="Pa")
    effective_diameter = convert_unit(effective_diameter, unit_from="in", unit_to="m")

    f_h = [(effective_diameter ** 2) * (pi/4) * whp] * len(tvd)
    f_h = [x / 1000 for x in f_h]  # N to kN

    return f_h


def pickup_force(tvd, od_csg, id_csg, tvd_toc, t_k, t_o, e, alpha):
    """
    Calculate axial force due to picking-up
    :param tvd: list - true vertical depth, m
    :param od_csg: pipe outer diameter, in
    :param id_csg: pipe inner diameter, in
    :param tvd_toc: tvd at top of cement, m
    :param t_k: max. wellhead temperature during production, °C
    :param t_o: mean ambient temperature, °C
    :param e: pipe Young's modulus, bar
    :param alpha: thermal expansion coefficient, 1/°C
    :return: axial force profile, kN
    """

    delta_t = [x - y for x, y in zip(t_k, t_o)]
    area = (pi/4) * (od_csg**2 - id_csg*2)
    area = convert_unit(area, unit_from="in2", unit_to="m2")
    e = convert_unit(e, unit_from="bar", unit_to="Pa")

    f_pu = [e * area * alpha * y for x, y in zip(tvd, delta_t) if x <= tvd_toc]
    f_pu += [0] * (len(tvd) - len(f_pu))
    f_pu = [x / 1000 for x in f_pu]  # N to kN

    return f_pu


def thermal_load(od_csg, id_csg, t_k, t_o, alpha, e):
    """
    Calculate axial force fue to thermal effect
    :param od_csg: pipe outer diameter, in
    :param id_csg: pipe inner diameter, in
    :param t_k: max. wellhead temperature during production, °C
    :param t_o: mean ambient temperature, °C
    :param alpha: thermal expansion coefficient, 1/°C
    :param e: pipe Young's modulus, bar
    :return: axial force profile, kN
    """

    delta_t = [x - y for x, y in zip(t_k, t_o)]
    area = (pi / 4) * (od_csg ** 2 - id_csg * 2)
    area = convert_unit(area, unit_from="in2", unit_to="m2")
    e = convert_unit(e, unit_from="bar", unit_to="Pa")

    f_th = [- e * area * alpha * x for x in delta_t]
    f_th = [x / 1000 for x in f_th]  # N to kN

    return f_th


''' This function needs to be checked!
def ballooning(md, md_toc, od_csg, id_csg, delta_rho_i, delta_rho_a, e, delta_p_i, delta_p_a, poisson=0.3):
    """
    Calculate axial force due to balloning
    :param md: list - measured depth, m
    :param md_toc: md at top of cement, m
    :param od_csg: pipe outer diameter, in
    :param id_csg: pipe inner diameter, in
    :param delta_rho_i: density change of fluid in the casing string after installation
    :param delta_rho_a: density change of annular fluid after installation
    :param e: pipe Young's modulus, bar
    :param delta_p_i: pressure change of fluid in the casing string after installation
    :param delta_p_a: pressure change of annular fluid after installation
    :param poisson: Poisson’s ratio
    :return: axial force profile, kN
    """

    e = convert_unit(e, unit_from="bar", unit_to="Pa")

    dia_rel = (od_csg/id_csg)**2
    delta_l = - poisson * (md_toc**2 / e) * ((delta_rho_i - dia_rel * delta_rho_a) / (dia_rel - 1)) -  \
        2 * poisson * (md_toc / e) * ((delta_p_i - dia_rel * delta_p_a) / (dia_rel - 1))

    area = (pi / 4) * (od_csg ** 2 - id_csg * 2)
    area = convert_unit(area, unit_from="in2", unit_to="m2")
    
    f_bl = [e * area * delta_l / md_toc for x in md if x <= md_toc]
    f_bl += [0] * (len(md) - len(f_bl))
    f_bl = [x / 1000 for x in f_bl]  # N to kN

    return f_bl'''


def shock_load(tvd, v_avg, od_csg, id_csg, nominal_weight, e, a=1.5):
    """
    Calculate axial force due to sudden stop during running
    :param tvd: list - true vertical depth, m
    :param v_avg: average running speed, m/s
    :param od_csg: pipe outer diameter, in
    :param id_csg: pipe inner diameter, in
    :param nominal_weight: weight per unit length, kg/m
    :param e: pipe Young's modulus, bar
    :param a: ratio of maximum running speed to average running speed
    :return: axial force profile, kN
    """

    area = (pi / 4) * (od_csg ** 2 - id_csg * 2)
    area = convert_unit(area, unit_from="in2", unit_to="m2")
    e = convert_unit(e, unit_from="bar", unit_to="Pa")

    rho_pipe = nominal_weight / area

    f_sh = [a * v_avg * area * (e * rho_pipe) ** 0.5] * len(tvd)
    f_sh = [x / 1000 for x in f_sh]  # N to kN

    return f_sh


def pressure_profile(tvd, tvd_fluid, rho_fluid):
    """
    Generate hydrostatic pressure profile
    :param tvd: list - true vertical depth, m
    :param tvd_fluid: list - reference tvd of fluid change, m
    :param rho_fluid: list - downwards sorted fluids densities, sg
    :return: pressure profile, Pa
    """
    g = 9.81        # gravity constant, [m/s2]
    rho_fluid = [convert_unit(x, unit_from="sg", unit_to="kg/m3") for x in rho_fluid]   # convert sg to kg/m3

    tvd_fluid.append(tvd[-1])

    p_prev = 0
    tvd_fluid_prev = 0
    rho_fluid = iter(rho_fluid)
    tvd_fluid = iter(tvd_fluid)
    rho_fluid_selected = next(rho_fluid)
    tvd_fluid_selected = next(tvd_fluid)

    pressure = []
    density = []
    for x in tvd:
        p = g * rho_fluid_selected * (x - tvd_fluid_prev) + p_prev
        density.append(rho_fluid_selected)

        if (x >= tvd_fluid_selected) and (tvd_fluid_selected != tvd[-1]):
            tvd_fluid_prev = tvd_fluid_selected
            tvd_fluid_selected = next(tvd_fluid)
            rho_fluid_selected = next(rho_fluid)
            p_prev = p
        pressure.append(p)

    return pressure


def density_profile(tvd, tvd_fluid, rho_fluid):
    """
    Generate a density profile from specific density values and depth of change
    :param tvd: list - true vertical depth, m
    :param tvd_fluid: list - reference tvd of fluid change, m
    :param rho_fluid: list - downwards sorted fluids densities, sg
    :return: density profile
    """

    tvd_fluid.append(tvd[-1])

    rho_fluid = iter(rho_fluid)
    tvd_fluid = iter(tvd_fluid)
    rho_fluid_selected = next(rho_fluid)
    tvd_fluid_selected = next(tvd_fluid)

    density = []
    for x in tvd:
        density.append(rho_fluid_selected)

        if (x >= tvd_fluid_selected) and (tvd_fluid_selected != tvd[-1]):
            tvd_fluid_selected = next(tvd_fluid)
            rho_fluid_selected = next(rho_fluid)

    return density



from numpy import linspace
from .collapse_calcs import calc_collapse_pressure


def api_limits(dt, yield_strength, limits, area, df_tension=1.3, df_compression=1.3, df_burst=1.1,
               df_collapse=1.1):

    # Defining respective limits according with design factors
    burst_limit = limits['burst'] / df_burst
    collapse_limit = limits['collapse'] / df_collapse
    tension_limit = limits['tension'] / df_tension
    compression_limit = limits['compression'] / df_compression

    # Zone with collapse and tension
    axial_force = linspace(0, tension_limit, 20)
    axial_stress = [x/area for x in axial_force]
    diff_pressure = [- calc_collapse_pressure(dt, yield_strength, x) / df_collapse for x in axial_stress]
    collapse_tens_line = [axial_force, diff_pressure]

    # Generating lines [x_list, y_list]
    burst_line = [[compression_limit, tension_limit], [burst_limit, burst_limit]]
    compression_line = [[compression_limit, compression_limit], [collapse_limit, burst_limit]]
    collapse_comp_line = [compression_limit, 0], [collapse_limit, collapse_limit]
    tension_line = [tension_limit, tension_limit], [burst_limit, collapse_tens_line[1][-1]]

    x_values = burst_line[0] + tension_line[0] + list(collapse_tens_line[0][::-1]) \
        + list(collapse_comp_line[0][::-1]) + compression_line[0]

    y_values = burst_line[1] + tension_line[1] + list(collapse_tens_line[1][::-1]) \
        + list(collapse_comp_line[1][::-1]) + compression_line[1]

    api_lines = [x_values, y_values]

    return api_lines, collapse_tens_line



from numpy import linspace
from math import pi


def vme(yield_s, area, int_diam, out_diam, design_factor=1.25):

    yield_s /= design_factor

    yield_s_new = yield_s * 1.1547

    xaxis_pos = linspace(0, yield_s_new, 300)
    xaxis_neg = linspace(- yield_s_new, 0, 300)[:-1]

    a_factor = ((pi * (out_diam/2)**2) + (pi * (int_diam/2)**2)) / area

    ellipse = [[x * area for x in xaxis_neg] + [x * area for x in xaxis_pos],
               [0.5 * (x + (4*yield_s**2 - 3*x**2)**0.5)/a_factor for x in xaxis_neg] +
               [0.5 * (x + (4*yield_s**2 - 3*x**2)**0.5)/a_factor for x in xaxis_pos],
               [0.5 * (x - (4*yield_s**2 - 3*x**2)**0.5)/a_factor for x in xaxis_neg] +
               [0.5 * (x - (4*yield_s**2 - 3*x**2)**0.5)/a_factor for x in xaxis_pos]]

    return ellipse



def get_conn_limits(limits, conn_compression=0.6, conn_tension=0.6, df_conn_compression=1.0,
                    df_conn_tension=1.0):

    tension_limit = limits['tension']
    compression_limit = limits['compression']

    conn_compression_limit = compression_limit * conn_compression / df_conn_compression
    conn_tension_limit = tension_limit * conn_tension / df_conn_tension

    return [conn_compression_limit, conn_tension_limit]



from ..unit_converter import convert_unit
g = 9.81        # gravity constant, [m/s2]


def onefluid_behindcasing(tvd, rho_mud):
    """
    Calculate external pressure profile with one fluid behind casing.
    :param tvd: list - true vertical depth, m
    :param rho_mud: mud density, sg
    :return: internal pressure profile, Pa
    """

    rho_mud = convert_unit(rho_mud, unit_from="sg", unit_to="kg/m3")

    p_ext = [g * rho_mud * x for x in tvd]

    return p_ext


def morefluids_behindcasing(tvd, rho_fluid, tvd_fluid):
    """
    Calculate external pressure profile with more than one fluid behind casing.
    :param tvd: list - true vertical depth, m
    :param rho_fluid: list - downwards sorted fluids densities, sg
    :param tvd_fluid: list - reference tvd of fluid change, m
    :return: internal pressure profile, Pa
    """

    tvd_fluid.append(tvd[-1])

    p_prev = 0
    tvd_fluid_prev = 0
    rho_fluid = iter(rho_fluid)
    tvd_fluid = iter(tvd_fluid)
    rho_fluid_selected = next(rho_fluid)
    tvd_fluid_selected = next(tvd_fluid)

    p_ext = []
    for x in tvd:
        rho_fluid_selected = convert_unit(rho_fluid_selected, unit_from="sg", unit_to="kg/m3")
        p = g * rho_fluid_selected * (x - tvd_fluid_prev) + p_prev

        if (x >= tvd_fluid_selected) and (tvd_fluid_selected != tvd[-1]):
            tvd_fluid_prev = tvd_fluid_selected
            tvd_fluid_selected = next(tvd_fluid)
            rho_fluid_selected = next(rho_fluid)
            p_prev = p
        p_ext.append(p)

    return p_ext


def injection(tvd, tvd_perf, p_inj, rho_inj, tvd_influencedzone, rho_fluid, p_fric, rho_form):
    p_ext = []
    for x in tvd:
        if x <= tvd_influencedzone:
            p_ext.append(rho_fluid * g * x)
        else:
            p_ext.append(p_inj + (rho_inj * g * tvd_perf) - p_fric - (rho_form * g * (tvd_perf - x)))

    return p_ext


def gas_migration(tvd, p_res, rho_mud, g):
    p_ext = [p_res + g * rho_mud * x for x in tvd]

    return p_ext



from ..unit_converter import convert_unit
g = 9.81        # gravity constant, [m/s2]


def onefluid_behindcasing(tvd, rho_mud):
    """
    Calculate external pressure profile with one fluid behind casing.
    :param tvd: list - true vertical depth, m
    :param rho_mud: mud density, sg
    :return: internal pressure profile, Pa
    """

    rho_mud = convert_unit(rho_mud, unit_from="sg", unit_to="kg/m3")

    p_ext = [g * rho_mud * x for x in tvd]

    return p_ext


def morefluids_behindcasing(tvd, rho_fluid, tvd_fluid):
    """
    Calculate external pressure profile with more than one fluid behind casing.
    :param tvd: list - true vertical depth, m
    :param rho_fluid: list - downwards sorted fluids densities, sg
    :param tvd_fluid: list - reference tvd of fluid change, m
    :return: internal pressure profile, Pa
    """
    rho_fluid = [convert_unit(x, unit_from="sg", unit_to="kg/m3") for x in rho_fluid]  # convert sg to kg/m3

    tvd_fluid.append(tvd[-1])

    p_prev = 0
    tvd_fluid_prev = 0
    rho_fluid = iter(rho_fluid)
    tvd_fluid = iter(tvd_fluid)
    rho_fluid_selected = next(rho_fluid)
    tvd_fluid_selected = next(tvd_fluid)

    p_ext = []
    for x in tvd:
        p = g * rho_fluid_selected * (x - tvd_fluid_prev) + p_prev

        if (x >= tvd_fluid_selected) and (tvd_fluid_selected != tvd[-1]):
            tvd_fluid_prev = tvd_fluid_selected
            tvd_fluid_selected = next(tvd_fluid)
            rho_fluid_selected = next(rho_fluid)
            p_prev = p
        p_ext.append(p)
    return p_ext


def depleted_zone(tvd, tvd_zone, p_zone, rho_mud):
    """
    Calculate external pressure profile with a depleted zone.
    :param tvd: list - true vertical depth, m
    :param tvd_zone: tvd at depleted zone, m
    :param p_zone: pressure at depleted zone, bar
    :param rho_mud: mud density, sg
    :return: internal pressure profile, Pa
    """

    p_zone = convert_unit(p_zone, unit_from="bar", unit_to="Pa")
    rho_mud = convert_unit(rho_mud, unit_from="sg", unit_to="kg/m3")

    tvd_mud_droplevel = tvd_zone - p_zone / (g * rho_mud)

    p_ext = []
    for x in tvd:
        if x <= tvd_mud_droplevel:
            p_ext.append(0)
        else:
            p_ext = g * rho_mud * (x - tvd_mud_droplevel)

    return p_ext



from ..unit_converter import convert_unit
g = 9.81        # gravity constant, [m/s2]


def fraction_of_bhp_at_wh(tvd, rho_mud, tvd_next_section, fraction=0.5):
    """
    Calculate internal pressure profile when a fraction of the bottom hole pressure (bhp) is at wellhead.
    :param tvd: list - true vertical depth, m
    :param rho_mud: mud density, sg
    :param tvd_next_section: tvd at bottom, m
    :param fraction: bhp fraction at wh
    :return: internal pressure profile, Pa
    """

    rho_mud = convert_unit(rho_mud, unit_from="sg", unit_to="kg/m3")
    bhp = g * rho_mud * tvd_next_section
    p_int = [fraction * bhp] * len(tvd)

    return p_int


def frac_shoe_gas_grad_above(tvd, frac_gradient, rho_fluid):
    """
    Calculate internal pressure profile when fracture pressure ar shoe with gas gradient above.
    :param tvd: list - true vertical depth, m
    :param frac_gradient: fracture gradient, bar/m
    :param rho_fluid: fluid density, sg
    :return: internal pressure profile, Pa
    """

    frac_gradient = convert_unit(frac_gradient, unit_from="bar", unit_to="Pa")      # from bar/m to Pa/m
    rho_fluid = convert_unit(rho_fluid, unit_from="sg", unit_to="kg/m3")
    tvd_frac = tvd[-1]
    p_frac = frac_gradient * tvd_frac
    p_int = [p_frac - g * rho_fluid * (tvd_frac - x) for x in tvd]

    return p_int


def gas_kick(tvd, rho_mud, kick_intensity, tvd_kick, vol_kick_initial, rho_kick_initial, id_csg, od_dp):
    """
    Calculate internal pressure profile when circulating out of a kick using the driller’s method.
    :param tvd: list - true vertical depth, m
    :param rho_mud: mud density, sg
    :param kick_intensity: required increase in mud density to control the kick, sg
    :param tvd_kick: tvd of influx, m
    :param vol_kick_initial: influx initial volume, m3
    :param rho_kick_initial: influx initial density, sg
    :param id_csg: casing inner diameter, in
    :param od_dp: drill pipe outer diameter, in
    :return: internal pressure profile, Pa
    """

    from math import pi
    from numpy import array

    rho_mud = convert_unit(rho_mud, unit_from="sg", unit_to="kg/m3")
    kick_intensity = convert_unit(kick_intensity, unit_from="sg", unit_to="kg/m3")
    rho_kick_initial = convert_unit(rho_kick_initial, unit_from="sg", unit_to="kg/m3")

    bhp = g * (rho_mud + kick_intensity) * tvd_kick     # bottom hole pressure [Pa]

    p = [g * rho_mud * x for x in tvd]      # hydrostatic pressure profile [Pa]

    vol_kick = [vol_kick_initial * (bhp / x) for x in p]  # * (temp/temp_kick) * (z/z_kick) [m3]
    rho_kick = [rho_kick_initial * (vol_kick_initial / x) for x in vol_kick]

    p_basekick = [bhp - g * rho_mud * (tvd_kick - x) for x in tvd]      # [Pa]

    ir_csg = convert_unit(id_csg/2, unit_from="in", unit_to="m")
    or_dp = convert_unit(od_dp/2, unit_from="in", unit_to="m")

    h = [x / (pi * (ir_csg ** 2 - or_dp ** 2)) for x in vol_kick]       # influx height [m]
    tvd_topkick = array([x - y for x, y in zip(tvd, h)])
    tvd_topkick[tvd_topkick < 0] = 0

    p_topkick = [a - g * b * (c - d) for a, b, c, d in zip(p_basekick, rho_kick, tvd, tvd_topkick) if d >= 0]

    whp = [x - g * rho_mud * y for x, y in zip(p_topkick, tvd_topkick)]

    p_int = [max(whp) + ((bhp - max(whp)) / tvd_kick) * x for x in tvd]

    return p_int


def displacement_to_gas(tvd, p_res, rho_gas, tvd_res):
    """
    Calculate internal pressure profile when the entire casing string is filled with gas.
    :param tvd: list - true vertical depth, m
    :param p_res: reservoir pressure, bar
    :param rho_gas: gas density, sg
    :param tvd_res: tvd at reservoir, m
    :return: internal pressure profile, Pa
    """

    p_res = convert_unit(p_res, unit_from="bar", unit_to="Pa")
    rho_gas = convert_unit(rho_gas, unit_from="sg", unit_to="kg/m3")

    p_int = [p_res - g * rho_gas * (tvd_res - x) for x in tvd]

    return p_int


def pressure_test(tvd, p_test, rho_mud, tvd_ref):
    """
    Calculate internal pressure profile based on mud weight applied pressure at the wellhead.
    :param tvd: list - true vertical depth, m
    :param p_test: testing pressure, bar
    :param rho_mud: list - downwards sorted mud densities, sg
    :param tvd_ref: list - reference tvd of fluid change, m
    :return: internal pressure profile, Pa
    """

    p_test = convert_unit(p_test, unit_from="bar", unit_to="Pa")

    rho_fluid = [convert_unit(x, unit_from="sg", unit_to="kg/m3") for x in rho_mud]  # convert sg to kg/m3

    tvd_ref.append(tvd[-1])

    p_prev = p_test
    tvd_fluid_prev = 0
    rho_fluid = iter(rho_fluid)
    tvd_fluid = iter(tvd_ref)
    rho_fluid_selected = next(rho_fluid)
    tvd_fluid_selected = next(tvd_fluid)

    p_ext = []
    for x in tvd:
        p = g * rho_fluid_selected * (x - tvd_fluid_prev) + p_prev

        if (x >= tvd_fluid_selected) and (tvd_fluid_selected != tvd[-1]):
            tvd_fluid_prev = tvd_fluid_selected
            tvd_fluid_selected = next(tvd_fluid)
            rho_fluid_selected = next(rho_fluid)
            p_prev = p
        p_ext.append(p)

    return p_ext


def tubing_leak(tvd, p_res, rho_fluid, tvd_perf, rho_packerfluid, tvd_packer, rho_mud):
    """
    Calculate internal pressure profile when a shut-in pressure applied to the top of the production
    annulus due to a tubing leak near the wellhead.
    :param tvd: list - true vertical depth, m
    :param p_res: reservoir pressure, bar
    :param rho_fluid: fluid density, sg
    :param tvd_perf: tvd at perforations, m
    :param rho_packerfluid: packer fluid density, sg
    :param tvd_packer: tvd at packer, m
    :param rho_mud: mud density, sg
    :return: internal pressure profile, Pa
    """

    p_res = convert_unit(p_res, unit_from="bar", unit_to="Pa")
    rho_fluid = convert_unit(rho_fluid, unit_from="sg", unit_to="kg/m3")
    rho_packerfluid = convert_unit(rho_packerfluid, unit_from="sg", unit_to="kg/m3")

    whp = p_res - g * rho_fluid * tvd_perf      # wellhead pressure [Pa]

    p_int = [whp + g * rho_packerfluid * x for x in tvd if x <= tvd_packer] + \
            [p_res - g * rho_packerfluid * (tvd_perf - x) for x in tvd if (x > tvd_packer) and (x <= tvd_perf)] + \
            [p_res + g * rho_mud * (x - tvd_perf) for x in tvd if x > tvd_perf]

    return p_int


def tubing_leak_stimulation(tvd, whp, rho_packerfluid, rho_injectionfluid, tvd_packer=0):
    """

    :param tvd: list - true vertical depth, m
    :param whp: wellhead pressure, bar
    :param rho_packerfluid: packer fluid density, sg
    :param rho_injectionfluid: injection fluid density, sg
    :param tvd_packer: tvd at packer, m
    :return: internal pressure profile, Pa
    """

    whp = convert_unit(whp, unit_from="bar", unit_to="Pa")
    rho_injectionfluid = convert_unit(rho_injectionfluid, unit_from="sg", unit_to="kg/m3")
    rho_packerfluid = convert_unit(rho_packerfluid, unit_from="sg", unit_to="kg/m3")

    p_int = [whp + g * rho_packerfluid * x for x in tvd if x <= tvd_packer] + \
            [whp + g * rho_injectionfluid * x for x in tvd if x > tvd_packer]

    return p_int



from math import pi
from .unit_converter import convert_unit
from .collapse_calcs import calc_collapse_pressure
from .von_mises import vme
from .design_factors import api_limits
from .connections import get_conn_limits


class Casing(object):
    """
    Casing object.

    Arguments:
        od_csg (outer diameter, float or int): casing outer diameter [in].
        id_csg (inner diameter, float or int): casing inner diameter [in].
        shoe_depth (depth at shoe, float or int): measured depth at shoe [m].

    Keyword Arguments:
        nominal_weight (nominal weight, float or int): weight per unit length [kg/m].
        yield_s (yield strength, int): pipe's yield strength [psi].
        df_tension (design factor - tension, float): API design factor for tension.
        df_compression (design factor - compression, float): API design factor for compression.
        df_burst (design factor - burst, float): API design factor for burst.
        df_collapse (design factor - collapse, float): API design factor for collapse.
        df_vme (design factor - Von Mises, float): design factor for triaxial.

    Attributes:
        od (float): outer diameter of the casing [in]
        id (float): inner diameter of the casing [in]
        thickness (str or None): outer diameter of the casing [in]
        dt (float): ratio --> outer diameter / thickness
        area (float): effective area [in^2]
        shoe_depth (float or int): measured depth at shoe [m]
        ellipse (list): triaxial points [x, y+, y-]
        csg_loads (list): list of loads that have been run
        nominal_weight (float or int): weight per unit length [kg/m]
        trajectory (obj): wellbore trajectory object
        api_lines (list): API limits coordinates [x, y]
        design_factor (dict): design factors used 'vme', 'api_compression', 'api_tension', 'api_burst', 'api_collapse'
    """

    def __init__(self, od_csg, id_csg, shoe_depth, nominal_weight=64, yield_s=80000, df_tension=1.1,
                 df_compression=1.1, df_burst=1.1, df_collapse=1.1, df_vme=1.25, conn_compression=0.6,
                 conn_tension=0.6, df_conn_compression=1.0, df_conn_tension=1.0):

        self.od = od_csg
        self.id = id_csg
        self.area = (pi / 4) * (self.od ** 2 - self.id ** 2)
        self.thickness = (self.od - self.id) / 2
        self.dt = self.od / self.thickness
        self.limits = {'burst': 0.875 * 2 * yield_s * self.thickness / self.od,
                       'burst_df': 0.875 * 2 * yield_s * self.thickness / self.od / df_burst,
                       'collapse': - calc_collapse_pressure(self.dt, yield_s),
                       'collapse_df': - calc_collapse_pressure(self.dt, yield_s) / df_collapse,
                       'compression': - yield_s * self.area,
                       'compression_df': - yield_s * self.area / df_compression,
                       'tension': yield_s * self.area,
                       'tension_df': yield_s * self.area / df_tension}

        self.shoe_depth = shoe_depth
        self.ellipse = vme(yield_s, self.area, self.id, self.od, df_vme)
        self.csg_loads = []
        self.nominal_weight = nominal_weight
        self.trajectory = None
        self.api_lines, self.collapse_curve = api_limits(self.dt, yield_s, self.limits, self.area, df_tension,
                                                         df_compression, df_burst, df_collapse)
        self.conn_limits = get_conn_limits(self.limits, conn_compression, conn_tension,
                                           df_conn_compression, df_conn_tension)
        self.design_factor = {'vme': df_vme,
                              'api_compression': df_compression,
                              'api_tension': df_tension,
                              'api_burst': df_burst,
                              'api_collapse': df_collapse}

    def running(self, tvd_fluid=None, rho_fluid=None, v_avg=0.3, e=32e6, fric=0.24, a=1.5):
        """
        Run load case: Running in hole

        Keyword Arguments:
            tvd_fluid (list or None): reference tvd of fluid change
            rho_fluid (list or None): downwards sorted fluids densities
            v_avg (float): average running speed, m/s
            e (int): pipe Young's modulus, bar
            fric (float): sliding friction factor pipe - wellbore
            a (float): ratio of maximum running speed to average running speed

        Returns:
            None. It adds the load case results in csg_loads as [load case name, axial_force, pressure_differential]
        """

        from .load_cases import running

        if tvd_fluid is None:
            tvd_fluid = []
        if rho_fluid is None:
            rho_fluid = [1.2]

        axial_force, pressure_differential = running(self.trajectory, self.nominal_weight, self.od, self.id,
                                                     self.shoe_depth, tvd_fluid, rho_fluid, v_avg, e, fric, a)

        axial_force = [x * 1000 / 4.448 for x in axial_force]   # kN to lbf

        self.csg_loads.append(
            ["Running", axial_force, pressure_differential]
        )

    def overpull(self, tvd_fluid=None, rho_fluid=None, v_avg=0.3, e=32e6, fric=0.24, a=1.5, f_ov=0):
        """
        Run load case: Overpull

        Keyword Arguments:
            tvd_fluid (list or None): reference tvd of fluid change
            rho_fluid (list or None): downwards sorted fluids densities
            v_avg (float): average running speed, m/s
            e (int): pipe Young's modulus, bar
            fric (float): sliding friction factor pipe - wellbore
            a (float): ratio of maximum running speed to average running speed
            f_ov (int or float): overpull force (often during freeing of stuck pipe), kN.

        Returns:
            None. It adds the load case results in csg_loads as [load case name, axial_force, pressure_differential]
        """

        from .load_cases import overpull

        if tvd_fluid is None:
            tvd_fluid = []
        if rho_fluid is None:
            rho_fluid = [1.2]

        axial_force, pressure_differential = overpull(self.trajectory, self.nominal_weight, self.od, self.id,
                                                      self.shoe_depth, tvd_fluid, rho_fluid, v_avg, e, fric, a, f_ov)

        axial_force = [x * 1000 / 4.448 for x in axial_force]  # kN to lbf

        self.csg_loads.append(
            ["Overpull", axial_force, pressure_differential]
        )

    def green_cement(self, tvd_fluid_int=None, rho_fluid_int=None, rho_cement=1.8,
                     f_pre=0, p_test=0):
        """
        Run load case: Green Cement Pressure test

        Keyword Arguments:
            rho_cement (float): cement density, sg
            tvd_fluid_int (list or None): reference tvd of fluid change inside, m
            rho_fluid_int (list or None): downwards sorted fluids densities inside, sg
            f_pre (int or float): pre-loading force applied to the casing string if necessary, kN
            p_test (int or float): testing pressure, psi

        Returns:
            None. It adds the load case results in csg_loads as [load case name, axial_force, pressure_differential]
        """

        from .load_cases import green_cement_pressure_test

        if tvd_fluid_int is None:
            tvd_fluid_int = []
        if rho_fluid_int is None:
            rho_fluid_int = [1.2]

        tvd = self.trajectory.tvd
        f_test = convert_unit(p_test * self.area, unit_from="lbf", unit_to="kN")
        p_test = convert_unit(p_test, unit_from="psi", unit_to="bar")

        axial_force, pressure_differential = green_cement_pressure_test(tvd, self.nominal_weight, self.od, self.id,
                                                                        rho_cement, tvd_fluid_int,
                                                                        rho_fluid_int, p_test, f_test, f_pre)

        pressure_differential = convert_unit(pressure_differential, unit_from="Pa", unit_to="psi")

        axial_force = [x * 1000 / 4.448 for x in axial_force]  # kN to lbf

        self.csg_loads.append(
            ["Green Cement", axial_force, pressure_differential]
        )

    def add_trajectory(self, trajectory):

        trajectory.md = [x for x in trajectory.md if x <= self.shoe_depth]
        trajectory.cells_no = len(trajectory.md)
        trajectory.tvd = trajectory.tvd[:trajectory.cells_no]
        trajectory.inclination = trajectory.inclination[:trajectory.cells_no]

        self.trajectory = trajectory

    def plot(self, plot_type='plotly'):
        from .plot import create_plotly_figure, create_pyplot_figure
        if plot_type == 'plotly':
            fig = create_plotly_figure(self)
        else:
            fig = create_pyplot_figure(self)

        return fig



import matplotlib.pyplot as plt
import plotly.graph_objects as go
from numpy import array, interp


def create_pyplot_figure(self):

    fig, ax = plt.subplots()

    # Plotting VME
    ax.plot(array(self.ellipse[0]) / 1000, array(self.ellipse[1]) / 1000, 'r', label='Triaxial ' +
                                                                                     str(self.design_factor['vme']))
    ax.plot(array(self.ellipse[0]) / 1000, array(self.ellipse[2]) / 1000, 'r')

    # Plotting API limits
    ax.plot(array(self.api_lines[0]) / 1000, array(self.api_lines[1]) / 1000, 'k', label='API')

    # Plotting connections limits
    # for compression
    ax.plot([self.conn_limits[0]/1000]*2,
            [self.limits['collapse_df']/1000, self.limits['burst_df']/1000],
            '0.55',
            label='Connection')

    # for tension
    ax.plot([self.conn_limits[1] / 1000] * 2,
            [self.limits['burst_df'] / 1000,
             interp(self.conn_limits[1], self.collapse_curve[0], self.collapse_curve[1]) / 1000],
            '0.55')

    # Plotting Loads
    for x in self.csg_loads:
        ax.plot(array(x[1]) / 1000, array(x[2]) / 1000, label=x[0])

    ax.set_xlabel('Axial Force, klb-f')
    ax.set_ylabel('Pressure Difference, kpsi')
    ax.ticklabel_format(style='plain')
    ax.legend(loc='lower right', fontsize='x-small')
    ax.grid()

    return fig


def create_plotly_figure(self):
    fig = go.Figure()

    # Plotting VME
    triaxial_x = array(self.ellipse[0] + self.ellipse[0][::-1])/1000
    triaxial_y = array(self.ellipse[1] + self.ellipse[2][::-1])/1000
    fig.add_trace(go.Scatter(x=triaxial_x, y=triaxial_y, line={'color': 'red'},
                             name='Triaxial ' + str(self.design_factor['vme'])))

    # Plotting API limits
    fig.add_trace(go.Scatter(x=array(self.api_lines[0]) / 1000, y=array(self.api_lines[1]) / 1000,
                             line={'color': 'black'}, name='API'))

    # Plotting connections limits
    # for compression
    fig.add_trace(go.Scatter(x=[self.conn_limits[0] / 1000] * 2,
                             y=[self.limits['collapse_df'] / 1000, self.limits['burst_df'] / 1000],
                             line={'color': 'gray'}, name='Connection', mode='lines'))
    # for tension
    fig.add_trace(go.Scatter(x=[self.conn_limits[1] / 1000] * 2,
                             y=[self.limits['burst_df'] / 1000,
                                interp(self.conn_limits[1], self.collapse_curve[0], self.collapse_curve[1]) / 1000],
                             line={'color': 'gray'}, name='Connection', mode='lines'))
    fig['data'][-1]['showlegend'] = False       # Hide last label (otherwise it'd show twice 'Connection')

    # Plotting Loads
    for x in self.csg_loads:
        fig.add_trace(go.Scatter(x=array(x[1]) / 1000, y=array(x[2]) / 1000, name=x[0]))

    fig.update_layout(
        xaxis_title='Axial Force, klb-f',
        yaxis_title='Pressure Difference, kpsi')

    return fig



g = 9.81        # gravity constant, [m/s2]


def inside_full(tvd, rho_fluid):
    p_int = [g * rho_fluid * x for x in tvd]

    return p_int


def partial_evacuation(tvd, rho_mud, tvd_zone, p_zone):
    tvd_mud_droplevel = tvd_zone - p_zone / (g * rho_mud)

    p_int = []
    for x in tvd:
        if x <= tvd_mud_droplevel:
            p_int.append(0)
        else:
            p_int = g * rho_mud * (x - tvd_mud_droplevel)

    return p_int


def full_evacuation(tvd):
    p_int = [0] * len(tvd)

    return p_int



from setuptools import setup
with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='pwploads',
    packages=['pwploads', 'pwploads.axial', 'pwploads.burst', 'pwploads.collapse'],
    version='0.2.0',
    license='LGPL v3',
    description='Load Cases for Well Design',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Pro Well Plan AS',
    author_email='juan@prowellplan.com',
    url='https://github.com/pro-well-plan/pwploads',
    keywords='well design',
    classifiers=['Programming Language :: Python :: 3',
                 'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
                 'Natural Language :: English',
                 'Topic :: Scientific/Engineering',
                 'Topic :: Software Development',
                 'Topic :: Software Development :: Libraries',
                 'Topic :: Utilities'],
    install_requires=['numpy', 'matplotlib', 'plotly']
)



def running(trajectory, nominal_weight, od_csg, id_csg, shoe_depth, tvd_fluid, rho_fluid, v_avg, e,
            fric=0.24, a=1.5):
    """
    Calculate axial load during running
    :param trajectory: wellpath object
    :param nominal_weight: weight per unit length, kg/m
    :param od_csg: pipe outer diameter, in
    :param id_csg: pipe inner diameter, in
    :param shoe_depth: measured depth at shoe, m
    :param tvd_fluid: list - reference tvd of fluid change
    :param rho_fluid: list - downwards sorted fluids densities
    :param v_avg: average running speed, m/s
    :param e: pipe Young's modulus, bar
    :param fric: sliding friction factor pipe - wellbore
    :param a: ratio of maximum running speed to average running speed
    :return: total axial force profile, kN
    """

    from .forces import air_weight, buoyancy_force, shock_load, drag

    f_w = air_weight(trajectory.tvd, nominal_weight)
    f_bu = buoyancy_force(trajectory.tvd, od_csg, id_csg, tvd_fluid, rho_fluid, tvd_fluid, rho_fluid)
    f_sh = shock_load(trajectory.tvd, v_avg, od_csg, id_csg, nominal_weight, e, a)
    f_d = drag(trajectory, od_csg, id_csg, shoe_depth, nominal_weight, tvd_fluid, rho_fluid, fric)
    # f_be --> bending

    force = [x1 - x2 + x3 - x4 for x1, x2, x3, x4 in zip(f_w, f_bu, f_sh, f_d)]
    # + f_be
    return force


def pulling(trajectory, nominal_weight, od_csg, id_csg, shoe_depth, tvd_fluid, rho_fluid, v_avg, e,
            fric=0.24, a=1.5, f_ov=0):
    """
    Calculate axial load during pulling
    :param trajectory: wellpath object
    :param nominal_weight: weight per unit length, kg/m
    :param od_csg: pipe outer diameter, in
    :param id_csg: pipe inner diameter, in
    :param shoe_depth: measured depth at shoe, m
    :param tvd_fluid: list - reference tvd of fluid change
    :param rho_fluid: list - downwards sorted fluids densities
    :param v_avg: average running speed, m/s
    :param e: pipe Young's modulus, bar
    :param fric: sliding friction factor pipe - wellbore
    :param a: ratio of maximum running speed to average running speed
    :param f_ov: overpull force (often during freeing of stuck pipe), kN.
    :return: total axial force profile, kN
    """

    from .forces import air_weight, buoyancy_force, shock_load, drag

    f_w = air_weight(trajectory.tvd, nominal_weight)
    f_bu = buoyancy_force(trajectory.tvd, od_csg, id_csg, tvd_fluid, rho_fluid, tvd_fluid, rho_fluid)
    f_sh = shock_load(trajectory.tvd, v_avg, od_csg, id_csg, nominal_weight, e, a)
    f_d = drag(trajectory, od_csg, id_csg, shoe_depth, nominal_weight, tvd_fluid, rho_fluid, fric, 'hoisting')
    # f_be --> bending

    force = [x1 - x2 + x3 + x4 + f_ov for x1, x2, x3, x4 in zip(f_w, f_bu, f_sh, f_d)]
    # + f_be

    return force


def cementation(tvd, nominal_weight, od_csg, id_csg, tvd_fluid_ext, rho_fluid_ext, tvd_fluid_int, rho_fluid_int,
                f_pre=0):
    """
    Calculate axial load during cementing
    :param tvd: list - true vertical depth, m
    :param nominal_weight: weight per unit length, kg/m
    :param od_csg: pipe outer diameter, in
    :param id_csg: pipe inner diameter, in
    :param tvd_fluid_ext: list - reference tvd of fluid change outside, m
    :param rho_fluid_ext: list - downwards sorted fluids densities outside, sg
    :param tvd_fluid_int: list - reference tvd of fluid change inside, m
    :param rho_fluid_int: list - downwards sorted fluids densities inside, sg
    :param f_pre: pre-loading force applied to the casing string if necessary, kN
    :return: total axial force profile, kN
    """

    from .forces import air_weight, buoyancy_force

    f_w = air_weight(tvd, nominal_weight)
    f_bu = buoyancy_force(tvd, od_csg, id_csg, tvd_fluid_ext, rho_fluid_ext, tvd_fluid_int, rho_fluid_int)
    # f_be --> bending

    force = [x1 - x2 + f_pre for x1, x2 in zip(f_w, f_bu)]
    # + f_be

    return force


def green_cement(tvd, nominal_weight, od_csg, id_csg, rho_cement, tvd_fluid_int, rho_fluid_int,
                 f_pre=0, f_h=0):
    """
    Calculate axial load during green cement pressure test
    :param tvd: list - true vertical depth, m
    :param nominal_weight: weight per unit length, kg/m
    :param od_csg: pipe outer diameter, in
    :param id_csg: pipe inner diameter, in
    :param rho_cement: cement density, sg
    :param tvd_fluid_int: list - reference tvd of fluid change inside, m
    :param rho_fluid_int: list - downwards sorted fluids densities inside, sg
    :param f_pre: pre-loading force applied to the casing string if necessary, kN
    :param f_h: pressure testing force, kN
    :return: total axial force profile, kN
    """

    from .forces import air_weight, buoyancy_force

    f_w = air_weight(tvd, nominal_weight)
    f_bu = buoyancy_force(tvd, od_csg, id_csg, [], [rho_cement], tvd_fluid_int, rho_fluid_int)
    # f_be --> bending

    force = [x1 - x2 + f_h + f_pre for x1, x2 in zip(f_w, f_bu)]
    # + f_be

    return force


def production(md, md_toc, od_csg, id_csg, delta_rho_i, delta_rho_a, e, delta_p_i, delta_p_a, poisson=0.3, f_setting=0):
    """
    Calculate axial load during production
    :param md: list - measured depth, m
    :param md_toc: md at top of cement, m
    :param od_csg: pipe outer diameter, in
    :param id_csg: pipe inner diameter, in
    :param delta_rho_i: density change of fluid in the casing string after installation
    :param delta_rho_a: density change of annular fluid after installation
    :param e: pipe Young's modulus, bar
    :param delta_p_i: pressure change of fluid in the casing string after installation
    :param delta_p_a: pressure change of annular fluid after installation
    :param poisson: Poisson’s ratio
    :param f_setting: hang off force of the casing string on slips or hangers, kN
    :return: total axial force profile, kN
    """

    #from .forces import ballooning

    # f_bl = ballooning(md, md_toc, od_csg, id_csg, delta_rho_i, delta_rho_a, e, delta_p_i, delta_p_a, poisson)
    # f_be --> bending
    f_bl = [0] * len(md)  # while ballooning load is being checked

    force = [f_setting + x for x in f_bl]
    # + f_be

    return force


def injection(md, md_toc, od_csg, id_csg, delta_rho_i, delta_rho_a, e, delta_p_i, delta_p_a, t_k, t_o, alpha,
              poisson=0.3, f_setting=0):
    """
    Calculate axial load during injection
    :param md: list - measured depth, m
    :param md_toc: md at top of cement, m
    :param od_csg: pipe outer diameter, in
    :param id_csg: pipe inner diameter, in
    :param delta_rho_i: density change of fluid in the casing string after installation
    :param delta_rho_a: density change of annular fluid after installation
    :param e: pipe Young's modulus, bar
    :param delta_p_i: pressure change of fluid in the casing string after installation
    :param delta_p_a: pressure change of annular fluid after installation
    :param t_k: max. wellhead temperature during production, °C
    :param t_o: mean ambient temperature, °C
    :param alpha: thermal expansion coefficient, 1/°C
    :param poisson: Poisson’s ratio
    :param f_setting: hang off force of the casing string on slips or hangers, kN
    :return: total axial force profile, kN
    """

    from .forces import thermal_load

    f_th = thermal_load(od_csg, id_csg, t_k, t_o, alpha, e)
    # f_bl = ballooning(md, md_toc, od_csg, id_csg, delta_rho_i, delta_rho_a, e, delta_p_i, delta_p_a, poisson)
    f_bl = [0] * len(f_th)      # while ballooning load is being checked
    # f_be --> bending

    force = [f_setting + x1 + x2 for x1, x2 in zip(f_bl, f_th)]
    # + f_be

    return force



from . import axial, burst, collapse


def running(trajectory, nominal_weight, od_csg, id_csg, shoe_depth, tvd_fluid, rho_fluid, v_avg, e,
            fric=0.24, a=1.5):
    """
    Load case: Running in hole
    :param trajectory: wellpath object
    :param nominal_weight: weight per unit length, kg/m
    :param od_csg: pipe outer diameter, in
    :param id_csg: pipe inner diameter, in
    :param shoe_depth: measured depth at shoe, m
    :param tvd_fluid: list - reference tvd of fluid change
    :param rho_fluid: list - downwards sorted fluids densities
    :param v_avg: average running speed, m/s
    :param e: pipe Young's modulus, bar
    :param fric: sliding friction factor pipe - wellbore
    :param a: ratio of maximum running speed to average running speed
    :return: total axial force profile [kN] and pressure difference [psi]
    """

    axial_force = axial.running(trajectory, nominal_weight, od_csg, id_csg, shoe_depth, tvd_fluid, rho_fluid, v_avg,
                                e, fric, a)

    pressure_differential = [0] * len(axial_force)

    return axial_force, pressure_differential


def overpull(trajectory, nominal_weight, od_csg, id_csg, shoe_depth, tvd_fluid, rho_fluid, v_avg, e,
             fric=0.24, a=1.5, f_ov=0):
    """
    Load case: Overpull
    :param trajectory: wellpath object
    :param nominal_weight: weight per unit length, kg/m
    :param od_csg: pipe outer diameter, in
    :param id_csg: pipe inner diameter, in
    :param shoe_depth: measured depth at shoe, m
    :param tvd_fluid: list - reference tvd of fluid change
    :param rho_fluid: list - downwards sorted fluids densities
    :param v_avg: average running speed, m/s
    :param e: pipe Young's modulus, bar
    :param fric: sliding friction factor pipe - wellbore
    :param a: ratio of maximum running speed to average running speed
    :param f_ov: overpull force (often during freeing of stuck pipe), kN.
    :return: total axial force profile [kN] and pressure difference [psi]
    """

    axial_force = axial.pulling(trajectory, nominal_weight, od_csg, id_csg, shoe_depth, tvd_fluid, rho_fluid, v_avg,
                                e, fric, a, f_ov)

    pressure_differential = [0] * len(axial_force)

    return axial_force, pressure_differential


# BURST CASES

def green_cement_pressure_test(tvd, nominal_weight, od_csg, id_csg, rho_cement, tvd_fluid_int,
                               rho_fluid_int, p_test, f_h, f_pre=0):
    """
    Load case: Green Cement
    :param tvd: list - true vertical depth, m
    :param nominal_weight: weight per unit length, kg/m
    :param od_csg: pipe outer diameter, in
    :param id_csg: pipe inner diameter, in
    :param rho_cement: cement density, sg
    :param tvd_fluid_int: list - reference tvd of fluid change inside, m
    :param rho_fluid_int: list - downwards sorted fluids densities inside, sg
    :param p_test: testing pressure, bar
    :param f_h: pressure testing force, kN
    :param f_pre: pre-loading force applied to the casing string if necessary, kN
    :return: total axial force profile [kN] and pressure difference [psi]
    """

    axial_force = axial.green_cement(tvd, nominal_weight, od_csg, id_csg, rho_cement, tvd_fluid_int,
                                     rho_fluid_int, f_pre, f_h)

    pressure_differential = burst.pressure_test_onefluid(tvd, p_test, rho_fluid_int, tvd_fluid_int, rho_cement)

    return axial_force, pressure_differential


def production_with_packer(md, md_toc, od_csg, id_csg, delta_rho_i, delta_rho_a, e, delta_p_i, delta_p_a, tvd,
                           rho_fluid, rho_mud, p_res, tvd_perf, rho_packerfluid, tvd_packer, poisson=0.3,
                           f_setting=0):

    axial_force = axial.production(md, md_toc, od_csg, id_csg, delta_rho_i, delta_rho_a, e, delta_p_i, delta_p_a,
                                   poisson, f_setting)

    pressure_differential = burst.production_with_packer(tvd, rho_fluid, rho_mud, p_res, tvd_perf, rho_packerfluid,
                                                         tvd_packer)

    return axial_force, pressure_differential


def production_with_packer_depleted_zone(md, md_toc, od_csg, id_csg, delta_rho_i, delta_rho_a, e, delta_p_i, delta_p_a,
                                         tvd, rho_fluid, rho_mud, p_res, tvd_perf, rho_packerfluid, tvd_packer,
                                         tvd_zone, p_zone, poisson=0.3, f_setting=0):

    axial_force = axial.production(md, md_toc, od_csg, id_csg, delta_rho_i, delta_rho_a, e, delta_p_i, delta_p_a,
                                   poisson, f_setting)

    pressure_differential = burst.production_with_packer_and_depletedzone(tvd, rho_fluid, rho_mud, p_res, tvd_perf,
                                                                          rho_packerfluid, tvd_packer, tvd_zone,
                                                                          p_zone)

    return axial_force, pressure_differential


def production_without_packer(md, md_toc, od_csg, id_csg, delta_rho_i, delta_rho_a, e, delta_p_i, delta_p_a, tvd, p_res,
                              rho_gas, tvd_res, rho_mud, poisson=0.3, f_setting=0):

    axial_force = axial.production(md, md_toc, od_csg, id_csg, delta_rho_i, delta_rho_a, e, delta_p_i, delta_p_a,
                                   poisson, f_setting)

    pressure_differential = burst.production_without_packer(tvd, p_res, rho_gas, tvd_res, rho_mud)

    return axial_force, pressure_differential


def production_without_packer_depleted_zone(md, md_toc, od_csg, id_csg, delta_rho_i, delta_rho_a, e, delta_p_i,
                                            delta_p_a, tvd, p_res, rho_gas, tvd_res, tvd_zone, p_zone, rho_mud,
                                            poisson=0.3, f_setting=0):

    axial_force = axial.production(md, md_toc, od_csg, id_csg, delta_rho_i, delta_rho_a, e, delta_p_i, delta_p_a,
                                   poisson, f_setting)

    pressure_differential = burst.production_without_packer_and_depletedzone(tvd, p_res, rho_gas, tvd_res, tvd_zone,
                                                                             p_zone, rho_mud)

    return axial_force, pressure_differential


def stimulation(md, md_toc, od_csg, id_csg, delta_rho_i, delta_rho_a, e, delta_p_i, delta_p_a, t_k, t_o, alpha, tvd,
                whp, rho_injectionfluid, rho_mud, rho_packerfluid, poisson=0.3, f_setting=0, tvd_packer=0):

    axial_force = axial.injection(md, md_toc, od_csg, id_csg, delta_rho_i, delta_rho_a, e, delta_p_i, delta_p_a, t_k,
                                  t_o, alpha, poisson, f_setting)

    pressure_differential = burst.stimulation(tvd, whp, rho_injectionfluid, rho_mud, rho_packerfluid, tvd_packer)

    return axial_force, pressure_differential


def fluid_storage(md, md_toc, od_csg, id_csg, delta_rho_i, delta_rho_a, e, delta_p_i, delta_p_a, t_k, t_o, alpha,
                  poisson, f_setting, tvd, frac_gradient, rho_mud, rho_fluid=None, tvd_fluid=None):

    axial_force = axial.injection(md, md_toc, od_csg, id_csg, delta_rho_i, delta_rho_a, e, delta_p_i, delta_p_a, t_k,
                                  t_o, alpha, poisson, f_setting)

    if (type(rho_fluid) == list) and (type(tvd_fluid) == list):
        pressure_differential = burst.fluid_storage_morefluids_behindcasing(tvd, frac_gradient, rho_fluid, tvd_fluid)

    else:
        pressure_differential = burst.fluid_storage_onefluid_behindcasing(tvd, rho_fluid, frac_gradient, rho_mud)

    return axial_force, pressure_differential


def fluid_storage_depleted_zone(md, md_toc, od_csg, id_csg, delta_rho_i, delta_rho_a, e, delta_p_i, delta_p_a, t_k, t_o,
                                alpha, tvd, frac_gradient, rho_fluid, tvd_zone, p_zone, rho_mud,
                                poisson=0.3, f_setting=0):

    axial_force = axial.injection(md, md_toc, od_csg, id_csg, delta_rho_i, delta_rho_a, e, delta_p_i, delta_p_a, t_k,
                                  t_o, alpha, poisson, f_setting)

    pressure_differential = burst.fluid_storage_depletedzone(tvd, frac_gradient, rho_fluid, tvd_zone, p_zone, rho_mud)

    return axial_force, pressure_differential


# BURST CASES

def drilling_losses(md, md_toc, od_csg, id_csg, delta_rho_i, delta_rho_a, e, delta_p_i, delta_p_a, t_k, t_o, alpha,
                    tvd, rho_mud, tvd_zone, p_zone, poisson=0.3, f_setting=0):

    axial_force = axial.injection(md, md_toc, od_csg, id_csg, delta_rho_i, delta_rho_a, e, delta_p_i, delta_p_a, t_k,
                                  t_o, alpha, poisson, f_setting)

    pressure_differential = collapse.drilling_losses(tvd, rho_mud, tvd_zone, p_zone)

    return axial_force, pressure_differential


def plug_cementation(tvd, nominal_weight, od_csg, id_csg, tvd_fluid_ext, rho_fluid_ext, tvd_fluid_int, rho_fluid_int,
                     rho_mud, f_pre=0, rho_fluid=None, tvd_fluid=None):

    axial_force = axial.cementation(tvd, nominal_weight, od_csg, id_csg, tvd_fluid_ext, rho_fluid_ext, tvd_fluid_int,
                                    rho_fluid_int, f_pre)

    if (type(rho_fluid) == list) and (type(tvd_fluid) == list):
        pressure_differential = collapse.plug_cementation_morefluids_behindcasing(tvd, rho_fluid, tvd_fluid)

    else:
        pressure_differential = collapse.plug_cementation_onefluid_behindcasing(tvd, rho_fluid, rho_mud)

    return axial_force, pressure_differential


def drill_stem_test(md, md_toc, od_csg, id_csg, delta_rho_i, delta_rho_a, e, delta_p_i, delta_p_a, tvd, rho_mud,
                    tvd_zone, p_zone, poisson=0.3, f_setting=0, evacuation="full"):

    axial_force = axial.production(md, md_toc, od_csg, id_csg, delta_rho_i, delta_rho_a, e, delta_p_i, delta_p_a,
                                   poisson, f_setting)

    pressure_differential = [0] * len(axial_force)

    if evacuation == "full":
        pressure_differential = collapse.drill_stem_test_fullevacuation(tvd, rho_mud)

    if evacuation == "partial":
        pressure_differential = collapse.drill_stem_test_partialevacuation(tvd, rho_mud, tvd_zone, p_zone)

    return axial_force, pressure_differential


def production_evacuation(tvd, rho_mud, md, md_toc, od_csg, id_csg, delta_rho_i, delta_rho_a, e, delta_p_i, delta_p_a,
                          poisson=0.3, f_setting=0):

    axial_force = axial.production(md, md_toc, od_csg, id_csg, delta_rho_i, delta_rho_a, e, delta_p_i, delta_p_a,
                                   poisson, f_setting)

    pressure_differential = collapse.production_fullevacuation(tvd, rho_mud)

    return axial_force, pressure_differential


def injection_evacuation(tvd, tvd_perf, rho_inj, p_inj, tvd_influencedzone, rho_fluid, p_fric, rho_form, evacuation,
                         p_zone, rho_fluid_before_cementation, rho_fluid_above_packer, md, md_toc, od_csg, id_csg,
                         delta_rho_i, delta_rho_a, e, delta_p_i, delta_p_a, t_k, t_o, alpha, poisson=0.3, f_setting=0):

    axial_force = axial.injection(md, md_toc, od_csg, id_csg, delta_rho_i, delta_rho_a, e, delta_p_i, delta_p_a, t_k,
                                  t_o, alpha, poisson, f_setting)

    pressure_differential = [0] * len(axial_force)

    if evacuation == "full":
        pressure_differential = collapse.injection_fulllevacuation(tvd, tvd_perf, rho_inj, p_inj, tvd_influencedzone,
                                                                   rho_fluid, p_fric, rho_form)

    if evacuation == "partial":
        pressure_differential = collapse.injection_partialevacuation(tvd, tvd_perf, rho_inj, p_inj, p_zone,
                                                                     tvd_influencedzone, rho_fluid_before_cementation,
                                                                     p_fric, rho_form, rho_fluid_above_packer)

    return axial_force, pressure_differential
